@@ -1,5 +1,13 @@
 EMPTY <- 'EMPTY'
 
+#' Check if name is bound to a non-lambda.r object
+is.bound <- function(name) {
+  if (! exists(name, inherits=TRUE)) return(FALSE)
+
+  o <- get(name, inherits=TRUE)
+  ! any(c('lambdar.fun','lambdar.type') %in% class(o))
+}
+
 # f(a,b) %::% A : B : C
 '%::%' <- function(signature, types)
 {
@@ -15,6 +23,9 @@ EMPTY <- 'EMPTY'
   tree <- list(args=NULL)
   args_expr <- parse_fun(it)
   name <- args_expr$token[1]
+  if (is.bound(name))
+    stop("Function name is already bound to non lambda.r object")
+
   if (nrow(args_expr) > 1)
     tree$args <- args_expr[2:nrow(args_expr),]
   tree$types <- parse_types(it, tree$args, text)
@@ -43,6 +54,9 @@ EMPTY <- 'EMPTY'
   tree <- list(args=NULL)
   args_expr <- parse_fun(it)
   name <- args_expr$token[1]
+  if (is.bound(name))
+    stop("Function name is already bound to non lambda.r object")
+
   if (nrow(args_expr) > 1)
     tree$args <- args_expr[2:nrow(args_expr),]
   guard_expr <- parse_guard(it)
@@ -55,6 +69,9 @@ EMPTY <- 'EMPTY'
   tree$def <- body_fn(tree$args, body_expr)
   tree$signature <- s.expr
   tree$body <- b.expr
+  tree$ellipsis <- idx_ellipsis(tree)
+  tree$fill.tokens <- clean_tokens(tree)
+  tree$fill.defaults <- clean_defaults(tree)
 
   add_variant(name, tree)
   options(keep.source=os$keep.source)
@@ -65,28 +82,70 @@ EMPTY <- 'EMPTY'
 .ERR_NO_MATCH <- "No match for function"
 .ERR_USE_FUNCTION <- "No valid function for"
 .ERR_ENSURE_FAILED <- "Assertion '%s' failed for args = %s and result = %s"
-NewObject <- function(type.name, ...)
+#NewObject <- function(type.name, ...)
+NewObject <- function(type.fn,type.name, ...)
 {
-  result <- UseFunction(type.name, ...)
+  result <- UseFunction(type.fn,type.name, ...)
+
   type <- gsub('"','', type.name)
   if (!type %in% class(result))
     class(result) <- c(type, class(result))
   result
 }
 
-UseFunction <- function(fn.name, ...)
+# Some timings
+# Baseline:
+# g <- function(x) x
+# system.time(for (i in 1:10000) g(i) )
+#  user  system elapsed
+# 0.004   0.000   0.003
+#
+# S3:
+# h <- function(x, ...) UseMethod("h")
+# h.default <- function(x, ...) x
+# system.time(for (i in 1:10000) h(i) )
+#  user  system elapsed
+# 0.035   0.001   0.035
+#
+# Lambda.r:
+# f(x) %as% x
+# system.time(for (i in 1:10000) { fn <- get('f', inherits=TRUE) })
+#  user  system elapsed
+# 0.017   0.000   0.018
+#
+# system.time(for (i in 1:10000) f(i) )
+#  user  system elapsed
+# 1.580   0.005   1.590
+# 0.622   0.005   0.628
+# 0.443   0.003   0.447
+# 0.407   0.000   0.408
+# 0.391   0.001   0.392
+# 0.384   0.001   0.386
+# 0.372   0.003   0.376
+# 0.347   0.001   0.347
+# 0.305   0.000   0.305
+UseFunction <- function(fn,fn.name, ...)
 {
-  fn <- get(fn.name, inherits=TRUE)
   result <- NULL
+  # u:0.007 s:0.002
   raw.args <- list(...)
+  # u:0.305 s:0.010
+  # u:0.096 s:0.002
+  # u:0.088 s:0.004
+  # u:0.082 s:0.000
   vs <- get_variant(fn,length(raw.args))
   if (is.null(vs) || length(vs) < 1)
     stop(use_error(.ERR_NO_MATCH,fn.name,raw.args))
+
   matched.fn <- NULL
   for (v in vs)
   {
-    full.args <- fill_args(raw.args, v)
+    # u:1.007 s:0.006
+    # u:0.106 s:0.001
+    # u:0.068 s:0.001
+    full.args <- fill_args(raw.args, v$fill.tokens, v$fill.defaults, v$ellipsis)
     if (is.null(full.args)) next
+    # u:0.019 s:0.003
     full.type <- get_type(fn,v$type.index)
     if (!check_types(full.type, full.args)) next
     if (is.null(v$guard)) { matched.fn <- v$def; break }
@@ -96,7 +155,8 @@ UseFunction <- function(fn.name, ...)
   if (is.null(matched.fn))
     stop(use_error(.ERR_USE_FUNCTION,fn.name,raw.args))
 
-  if (attr(fn,'debug')) debug(matched.fn)
+  # u:0.058 s:0.006
+  if (is.debug(fn.name)) debug(matched.fn)
   result <- do.call(matched.fn, full.args)
 
   if (!is.null(full.type))
@@ -124,17 +184,28 @@ UseFunction <- function(fn.name, ...)
 }
 
 
-has_ellipsis <- function(tree) {
-  '...' %in% tree$args$token
+idx_ellipsis <- function(tree) {
+  which(tree$args$token == '...')
 }
 
-fill_args <- function(raw.args, tree)
-{
-  if (is.null(tree$args)) return(list())
+clean_tokens <- function(tree) {
+  if (length(tree$ellipsis) == 0)
+    tree$args$token
+  else
+    tree$args$token[-tree$ellipsis]
+}
 
-  has.ellipsis <- has_ellipsis(tree)
-  tree$args <- tree$args[tree$args$token != '...',]
-  defaults <- tree$args$default
+clean_defaults <- function(tree) {
+  if (length(tree$ellipsis) == 0)
+    tree$args$default
+  else
+    tree$args$default[-tree$ellipsis]
+}
+
+#fill_args <- function(raw.args, args, ellipsis)
+fill_args <- function(raw.args, tokens, defaults, ellipsis)
+{
+  if (is.null(args)) return(list())
 
   # This is for unnamed arguments
   if (is.null(names(raw.args)))
@@ -147,11 +218,11 @@ fill_args <- function(raw.args, tree)
   }
   else
   {
-    if (! has.ellipsis && any(! names(raw.args) %in% c("",tree$args$token))) 
+    if (length(ellipsis) == 0 && any(! names(raw.args) %in% c("",tokens))) 
       return(NULL)
     ds <- lapply(defaults, function(x) x)
-    names(ds) <- tree$args$token
-    shim <- tree$args$token[1:length(raw.args)]
+    names(ds) <- tokens
+    shim <- tokens[1:length(raw.args)]
     names(raw.args)[names(raw.args) == ""] <- shim[names(raw.args) == '']
     ds[names(raw.args)] <- raw.args
     gaps <- ! names(ds) %in% names(raw.args)
@@ -178,7 +249,7 @@ check_types <- function(raw.types, raw.args)
       if (! the.type %in% letters) return(the.type)
 
       if (is.null(type.map[[the.type]])) {
-        if (arg.types[[x]] %in% type.map)
+        if (any(arg.types[[x]] %in% type.map))
           type.map[[the.type]] <<- paste("!",arg.types[[x]],sep='')
         # Add the new type if it doesn't exist
         else
@@ -192,9 +263,9 @@ check_types <- function(raw.types, raw.args)
   }
 
   if (!is.null(ncol(arg.types)) && ncol(arg.types) > 0)
-    all(sapply(idx, function(x) declared.types[x] %in% arg.types[,x]))
+    all(sapply(idx, function(x) any(declared.types[[x]] %in% arg.types[,x])))
   else
-    all(sapply(idx, function(x) declared.types[x] %in% arg.types[[x]]))
+    all(sapply(idx, function(x) any(declared.types[[x]] %in% arg.types[[x]])))
 }
 
 # Get the return type of a function declaration. This is aware of type
@@ -577,14 +648,15 @@ add_variant <- function(fn.name, tree)
   args <- NULL
 
   if (is.null(tree$args))
-    tree$accepts <- 0
+    tree$accepts <- c(0,0)
   else {
     args <- tree$args
     required.args <- length(args$default[is.na(args$default)])
     if ('...' %in% tree$args$token)
-      tree$accepts <- c(required.args : nrow(args) - 1, Inf)
+      tree$accepts <- c(required.args-1, Inf)
+      #tree$accepts <- c(required.args : nrow(args) - 1, Inf)
     else
-      tree$accepts <- required.args : nrow(args)
+      tree$accepts <- c(required.args, nrow(args))
     type.index <- get_type_index(fn, nrow(args), active.type)
     if (!is.null(type.index) && length(type.index) > 0)
       tree$type.index <- type.index
@@ -603,10 +675,14 @@ add_variant <- function(fn.name, tree)
 
 get_variant <- function(fn, arg.length)
 {
+  # u:0.007 s:0.000
   raw <- attr(fn,'variants')
-  match.fn <- function(x)
-    arg.length >= min(x$accepts) & arg.length <= max(x$accepts)
-  matches <- sapply(raw, match.fn)
+  len <- length(raw)
+  matches <- vector(length=len)
+  for (j in 1:len) {
+    accepts <- raw[[j]]$accepts
+    matches[j] <- arg.length >= accepts[1] && arg.length <= accepts[2]
+  }
   raw[matches]
 }
 
@@ -653,10 +729,10 @@ add_type <- function(fn.name, tree)
   types <- attr(fn,'types')
 
   if (is.null(tree$args))
-    tree$accepts <- 0
+    tree$accepts <- c(0,0)
   else {
     args <- tree$args
-    tree$accepts <- length(args$default[is.na(args$default)]) : nrow(args)
+    tree$accepts <- c(length(args$default[is.na(args$default)]), nrow(args))
   }
   f <- function(x) {
     ifelse(types[[x]]$signature == tree$signature, x, NA)
@@ -731,17 +807,16 @@ setup_parent <- function(parent, where)
 init_function <- function(name)
 {
   if (is.type(name)) 
-    pattern <- 'function(...) NewObject("%s",...)'
+    pattern <- 'function(...) NewObject(%s,"%s",...)'
   else
-    pattern <- 'function(...) UseFunction("%s",...)'
-  fn <- eval(parse(text=sprintf(pattern,name)))
+    pattern <- 'function(...) UseFunction(%s,"%s",...)'
+  fn <- eval(parse(text=sprintf(pattern,name,name)))
   if (is.type(name))
     attr(fn, 'class') <- c('lambdar.type',attr(fn,'class'))
   else
     attr(fn, 'class') <- c('lambdar.fun',attr(fn,'class'))
   attr(fn, 'variants') <- list()
   attr(fn, 'types') <- list()
-  attr(fn, 'debug') <- FALSE
   fn
 }
 
